@rem =========================================================================
@rem  Copyright (C) GemTalk Systems 1986-2013.  All Rights Reserved..
@rem 
@rem  Name - convDbf
@rem 
@rem  Purpose - convert a GemStone extent to the current DBF version for upgrade
@rem 
@rem  Requirements - The following environment variables must be set:
@rem        GEMSTONE	   - set to the GemStone 6.x product tree
@rem        upgradeLogDir  - a writable directory for log files
@rem
@rem  Arguments - Takes one argument, the name of the dbf file to be converted
@rem 
@rem =========================================================================

@setlocal

@if not "%GEMSTONE%" == "" goto gemstoneok
  @echo ERROR: GemStone scripts require a GEMSTONE environment variable.
  @echo        Please set it to the directory where GemStone 6.x resides.
  @goto failed
:gemstoneok

@if exist %GEMSTONE%\bin\convDbf.cmd goto gsverok
  @echo ERROR: %GEMSTONE% is not is not a GemStone 6.x installation.
  @echo        Please set GEMSTONE to your GemStone 6.x installation.
  @goto failed
:gsverok

@if not "%upgradeLogDir%" == "" goto upgradelogok
  @echo ERROR: The environment variable upgradeLogDir has not been set.
  @echo        Please set upgradeLogDir to a writable directory.
  @goto failed
:upgradelogok

@if exist %upgradeLogDir% goto logdirok
  @echo ERROR: The directory %upgradeLogDir% does not exist!
  @echo        Please set upgradeLogDir to a writable directory.
  @goto failed
:logdirok

@set logfile="%upgradeLogDir%\conv.log"
@echo Begining conversion of %1.... 
@echo Logfile is %logfile%.

@rem =======================================
@rem check for existence of extent with copydbf -i

@rem echo "output from: copydbf -i %1 " >> %logfile%
@%GEMSTONE%\bin\copydbf -i %1 >> %logfile%
@if %ERRORLEVEL% EQU 0 goto dbfok
  @echo ERROR: unable to open %1 for read with copydbf -i.
  @echo        %1 must be the name of a GemStone extent file.
  @goto failed
:dbfok

@set inputfile="%upgradeLogDir%\conv.inp"

@rem =======================================
@rem No here document in *.cmd, so create an input file...

@echo '%1' opendbf > %inputfile%
@echo 1 readpage >> %inputfile%
@echo convertrootpage >> %inputfile%
@echo 1 writepage >> %inputfile%
@echo 17 readpage >> %inputfile%
@echo convertrootpage >> %inputfile%
@echo 17 writepage >> %inputfile%
@echo 33 readpage >> %inputfile%
@echo convertrootpage >> %inputfile%
@echo 33 writepage >> %inputfile%
@echo 49 readpage >> %inputfile%
@echo convertrootpage >> %inputfile%
@echo 49 writepage >> %inputfile%
@echo closedbf >> %inputfile%
@echo exit >> %inputfile%

@rem ======================================
@if exist %inputfile% goto inputok
  @echo ERROR: Failed to create a temporary input file. Please make sure
  @echo        %upgradeLogDir% is a writable directory.
  @goto failed
:inputok
@rem Do the conversion
@echo Converting %1.... >> %logfile%
@%GEMSTONE%\bin\pgsvrslow < %inputfile% >> %logfile%

@del %inputfile%

@if %ERRORLEVEL% EQU 0 goto checklog
  @echo ERROR: conversion failure on %1!
  @echo Please check that %1 is a GemStone extent file.
  @echo Check the file %logfile% for more information, 
  @echo  or run   copydbf -i  on the extent to check its type.
  @goto failed
:checklog

@if exist %logfile% goto convok
  @echo ERROR: Failed to write a log file. Please make sure
  @echo        %upgradeLogDir% is a writable directory.
  @goto failed
:convok

@echo Successful conversion of %1!

:failed
@endlocal
